@@ -204,6 +204,7 @@ static int FaxPrepareReceivedTiff(struct fax *fax)
 	if (ret) {
 		fax->b3transfer_error = errno;
 		wprint("NCCI %06x: Cannot fstat received file %s - %s\n", ncci->ncci, fax->file_name, strerror(fax->b3transfer_error));
+		close(fd);
 		return -fax->b3transfer_error;
 	}
 	fax->b3data_size = fst.st_size;
@@ -211,6 +212,7 @@ static int FaxPrepareReceivedTiff(struct fax *fax)
 	if (!fax->b3data) {
 		fax->b3transfer_error = errno;
 		wprint("NCCI %06x: Cannot map received file %s - %s\n", ncci->ncci, fax->file_name, strerror(fax->b3transfer_error));
+		close(fd);
 		return -fax->b3transfer_error;
 	}
 	fax->b3data_mapped = 1;
@@ -567,16 +569,22 @@ static struct fax *mFaxCreate(struct BInstance	*bi)
 	time_t cur_time;
 	struct mISDN_ctrl_req creq;
 
-
+	pthread_mutex_lock(&bi->lp->lock);
+	nf = bi->b3data;
+	if (nf) {
+		dprint(MIDEBUG_NCCI, "PLCI %04x: already created NCCI: %06x\n", bi->lp->plci, nf->ncci->ncci);
+		pthread_mutex_unlock(&bi->lp->lock);
+		return nf;
+	}
 	nf = calloc(1, sizeof(*nf));
 	if (nf) {
 		nf->file_d = -2;
 		nf->ncci = ncciCreate(bi->lp);
 		if (nf->ncci) {
-			dprint(MIDEBUG_NCCI, "NCCI %06x: create\n", nf->ncci->ncci);
 			nf->binst = bi;
 			nf->lplci = bi->lp;
 			nf->outgoing = nf->lplci->PLCI->outgoing;
+			dprint(MIDEBUG_NCCI, "NCCI %06x: create %s\n", nf->ncci->ncci, nf->outgoing ? "outgoing" : "incoming");
 			if (nf->lplci->Bprotocol.B1cfg[0])
 				nf->rate = CAPIMSG_U16(nf->lplci->Bprotocol.B1cfg, 1);
 			if (nf->lplci->Bprotocol.B3cfg[0]) {
@@ -617,7 +625,8 @@ static struct fax *mFaxCreate(struct BInstance	*bi)
 			if (ret < 0)
 				wprint("Error on MISDN_CTRL_RX_BUFFER  ioctl - %s\n", strerror(errno));
 			else
-				dprint(MIDEBUG_NCCI, "MISDN_CTRL_RX_BUFFER  old values: min=%d max=%d\n", creq.p1, creq.p2);
+				dprint(MIDEBUG_NCCI, "NCCI %06x: MISDN_CTRL_RX_BUFFER  values: min=%d -> %d max=%d\n",
+					nf->ncci->ncci, creq.p1, DEFAULT_PKT_SIZE, creq.p2);
 			nf->startdownlink = 1;
 		} else {
 			eprint("Cannot create NCCI for PLCI %04x\n", bi->lp ? bi->lp->plci : 0xffff);
@@ -626,6 +635,7 @@ static struct fax *mFaxCreate(struct BInstance	*bi)
 		}
 	}
 	bi->b3data = nf;
+	pthread_mutex_unlock(&bi->lp->lock);
 	return nf;
 }
 
@@ -1194,9 +1204,7 @@ int FaxB3Message(struct BInstance *bi, struct mc_buf *mc)
 		if (cmd == CAPI_CONNECT_B3 && subcmd == CAPI_REQ) {
 			fax = mFaxCreate(bi);
 		}
-		if (fax) {
-			bi->b3data = fax;
-		} else {
+		if (!fax) {
 			wprint("No Fax struct assigned\n");
 			return CapiMsgOSResourceErr;
 		}
